; slide-scanner-black-white-processing.scm
; version 2.0 [ziphra.narod.ru]
; last modified/tested by Alexey Rodionov
; 11/12/2011 on GIMP-2.6.11
;
; ------------------------------------------------------------------
; Original information ---------------------------------------------
;
; slide-scanner-black-white-processing image script  for GIMP 2.6
; Copyright (C) 2011 Alexey Rodionov <ziphra@yandex.ru>
; 
; End original information ------------------------------------------
;-------------------------------------------------------------------

   ; Вспомогательная подпрограмма, задающая имя нового обработанного файла   

(define (morph-filename orig-name new-extension)
 (let* ((buffer (vector "" "" "")))
  (if (re-match "^(.*)[.]([^.]+)$" orig-name buffer)
   (string-append (substring orig-name 0 (car (vector-ref buffer 2))) new-extension)
  )
 )
)

        ; Основная программа

(define (slide-scanner-black-white-processing InFolder inOutFolder width height x y bik brightness contrast neg flip format)
 
               ; Объявление переменных

 (let* ((filelist (cadr (file-glob (string-append InFolder DIR-SEPARATOR "*") 1))))
       
       (while (not (null? filelist))
               (let* ((filename (car filelist))
                      (image (car (gimp-file-load RUN-NONINTERACTIVE filename filename)))
                      (layer (car (gimp-image-get-active-layer image)))
                      (drawable (car (gimp-image-get-active-drawable image)))
                      (filename-negative)
                      )
                
                     (gimp-crop image width height x y)                                    ;обрезает изображение по указанным параметрам     
                     (gimp-image-scale image 3000 2000)                                    ; каркас негатива масштабируется до стандартного размера
                     (if (= flip TRUE) (gimp-image-flip image 0))                          ; переворачивает изображение горизонтально, использовано для восстановление изображения при сканировании со стороны фотоэмульсии

               ; Cохранение негатива

                     (if (= neg TRUE) (begin                                         
                                            (if (= format 0) (begin
                                                                   (set! filename-negative (string-append inOutFolder DIR-SEPARATOR "негатив-" (morph-filename (car (gimp-image-get-name image)) "jpg"))) ; определяется имя негатива изображения для последующего сохранения в виде "негатив-ХХХХХХХХ.jpg
                                                                   (file-jpeg-save RUN-NONINTERACTIVE image layer filename-negative filename-negative 0.9 0 1 1 "" 2 1 0 0) ;сохраняет файл в формате jpeg
                                                             )
                                             )
                                           (if (= format 1) (begin
                                                                   (set! filename-negative (string-append inOutFolder DIR-SEPARATOR "негатив-" (morph-filename (car (gimp-image-get-name image)) "tif"))) ; определяется имя негатива изображения для последующего сохранения в виде "негатив-ХХХХХХХХ.tiff              
                                                                  (file-tiff-save RUN-NONINTERACTIVE image layer filename-negative filename-negative 0) ; сохраняет негатив фото в формате tiff без компрессии
                                                            )
                                           )
                                     )
                     )

                 ; Обработка изображения

                     (gimp-invert drawable)
                     (set! drawable (car (gimp-image-get-active-drawable image)))
                     (gimp-levels-stretch drawable)        ;цвета растягиваются по динамическому диапазону для устранения остаточных искажений цвета и увеличения контраста
                     (if (= bik TRUE) (gimp-brightness-contrast drawable brightness contrast)) ; регулировка яркости и контраста, если выбрано
                     (gimp-desaturate drawable)            ;обесцвечивает фото, на случай если будут проявляться цвета
                     (set! layer (car (gimp-image-get-active-layer image)))

                 ; Сохранение результата обработки

                     (if (= format 0) (begin
                               (set! filename (string-append inOutFolder DIR-SEPARATOR (morph-filename (car (gimp-image-get-name image)) "jpg"))) ; определяется имя выходного файла и его размещение
                               (file-jpeg-save RUN-NONINTERACTIVE image layer filename filename 0.9 0 1 1 "" 2 1 0 0) ;сохраняет файл в формате jpeg
                          )
                     )
                    (if (= format 1) (begin                                       
                               (set! filename (string-append inOutFolder DIR-SEPARATOR (morph-filename (car (gimp-image-get-name image)) "tif"))) ; определяется имя выходного файла и его размещение
                               (file-tiff-save RUN-NONINTERACTIVE image layer filename filename 0) ; сохраняет негатив фото в формате tiff без компрессии
                          )
                    )


                    (gimp-image-delete image)          ; удаление переменной image
            )
           (set! filelist (cdr filelist))       ; удаление только что обработанного файла из списка filelist путём присвоения списку значения без первого файла
      )
  )
)

(script-fu-register
 "slide-scanner-black-white-processing"
"<Image>/Script-Fu/Мои скрипты/Обработка чёрно-белых негативов"
"Этот скрипт используется при оцифровке негативных чёрно-белых изображений фотоплёнки. Он обрабатывает целую папку негативов и сохраняет готовые фотографии в другую папку." 
;В обработку для чёрно-белых изображений входит:
;- обрезка ненужных краёв;
;- изменение размера (приведение его к стандартному);
;- переворот изображения (при сканировании со стороны эмульсии);
;- сохранение негатива (по желанию);
;- инвертирование (при обработке негативов, и, если в фотоаппарате нет функции негатив, сразу инвертирующей отснятое);
;- растяжение динамического диапазона для компенсации затемняющего эффекта фотоплёнки;
;- корректировка яркости и контраста изображения (при необходимости);
;- обесцвечивание (после растяжения динамического диапазона возможно появление какого-либо цвета);
;- сохранение готового изображения в форматах jpeg и tif."
 "Rodionov"
 "Copyleft, use it at your own sweet will"
 "11 декабря 2011"
 ""
 SF-DIRNAME "Исходная папка" "/ваша/исходная/папка_по_умолчанию"
 SF-DIRNAME "Папка назначения" "/ваша/назначения/папка_по_умолчанию"
 SF-ADJUSTMENT "ширина" '(4438 0 6000 1 10 0 SF-SPINNER)
 SF-ADJUSTMENT "высота" '(2959 0 6000 1 10 0 SF-SPINNER)
 SF-ADJUSTMENT "отступ слева" '(70 0 6000 1 10 0 SF-SPINNER)
 SF-ADJUSTMENT "отступ сверху" '(171 0 6000 1 10 0 SF-SPINNER)
 SF-TOGGLE "регулировка яркости и контраста" FALSE                  ; переменная bik
 SF-ADJUSTMENT "яркость" '(0 -100 100 1 10 0 0)
 SF-ADJUSTMENT "контрастность" '(0 -100 100 1 10 0 0)
 SF-TOGGLE "нужны негативы" FALSE                                   ; переменная neg
 SF-TOGGLE "отсканировано со стороны эмульсии" TRUE                 ; переменная flip
 SF-OPTION "сохранить в формате:" '("JPEG" "TIFF")                  ; переменная format
)



