; slide-scanner-color-foto-processing.scm
; version 2.5 [ziphra.narod.ru]
; last modified/tested by Alexey Rodionov
; 11/12/2011 on GIMP-2.6.11
;
; ------------------------------------------------------------------
; Original information ---------------------------------------------
;
; slide-scanner-color-foto-processing image script  for GIMP 2.6
; Copyright (C) 2011 Alexey Rodionov <kazfrend@mail.ru>
; 
; End original information ------------------------------------------
;-------------------------------------------------------------------

      ; Вспомогательная подпрограмма, задающая имя нового обработанного файла   

(define (morph-filename orig-name new-extension)
 (let* ((buffer (vector "" "" "")))
  (if (re-match "^(.*)[.]([^.]+)$" orig-name buffer)
   (string-append (substring orig-name 0 (car (vector-ref buffer 2))) new-extension)
  )
 )
)

        ; Основная программа

(define (slide-scanner-color-foto-processing InFolder inOutFolder Maska width height x y bik brightness contrast neg flip format)

        ; Объявление переменных

  (let* ((filelist (cadr (file-glob (string-append InFolder DIR-SEPARATOR "*") 1))) ;определяется список файлов во входящей папке (переменная InFolder)
         (maskImageOne (car (gimp-file-load RUN-NONINTERACTIVE Maska Maska)))       ; задаётся переменная maskImageOne - каркас маски-негатива
         (maskNegative (car (gimp-image-get-active-drawable maskImageOne)))         ; задаётся переменная maskNegative - изображение маски-негатива
         (maskImageTwo)                                                             ; задаётся переменная maskImageTwo - каркас маски-позитива
         (maskPositive)                                                             ; задаётся переменная maskPositive - изображение маски-позитива
        )

        ; Приготовление масок негатива (maskImageOne) и позитива (maskPositive)

        (gimp-image-crop maskImageOne width height x y)                             ; обрезаются ненужные края каркаса маски-негатива
        (gimp-image-scale maskImageOne 3000 2000)                                   ; каркас маски-негатива масштабируется до стандартного размера
        (if (= flip TRUE) (gimp-image-flip maskImageOne 0))                          ; переворачивает изображение горизонтально, использовано для восстановление изображения при сканировании со стороны фотоэмульсии
        (set! maskImageTwo (car (gimp-image-duplicate maskImageOne)))               ; каркас маски-позитива определяется как каркас маски-негатива
        (set! maskPositive (car (gimp-image-get-active-drawable maskImageTwo)))     ; изображение маски-позитива определяется как изображение маски-негатива

        (gimp-invert maskPositive)                                                  ; каркас и изображение маски-позитива стал теперь стали действительно                          позитивными  
 
                      ; Обработка изображения

        (while (not (null? filelist))                                                     ;задаётся цикл - условие: пуст ли список filelist?

                      ; Объявление переменных

               (let* ((filename (car filelist))                                           ;задаётся переменная filename - имя файла, первого в списке filelist
                      
                      (image (car (gimp-file-load RUN-NONINTERACTIVE filename filename))) ;переменная image - загружаемый файл с именем filename
                      (layer (car (gimp-image-get-active-layer image)))                   ;задаётся переменная layer - изображением из открытого файла в виде слоя
                      (drawable (car (gimp-image-get-active-drawable image)))             ;задаётся переменная drawable - изображение из открытого файла

                      (maskPositiveLayer (car (gimp-layer-new-from-drawable maskPositive image)));определяется переменная maskPositiveLayer - изображение маски-позитива в виде слоя для последующего добавления в image
                      (filename-negative)
                      (imageWhite)
                      (layerWhite)
                      (drawableWhite)
                      (maskNegativeLayer) ; роль последних переменных будет определена позже

                     )
                     
                     ; Подготовка негатива фото к обработке, 

                     (gimp-image-crop image width height x y)       ;обрезаются лишние края негатива
                     (gimp-image-scale image 3000 2000)             ;размер изображения приводится к стандартному 3000ч2000
                     (if (= flip TRUE) (gimp-image-flip image 0))   ; переворачивает изображение горизонтально, использовано для восстановление изображения при сканировании со стороны фотоэмульсии

                     ; Cохранение негатива

                     (if (= neg TRUE) (begin                                         
                                        (if (= format 0) (begin
                                                          (set! filename-negative (string-append inOutFolder DIR-SEPARATOR "негатив-" (morph-filename (car (gimp-image-get-name image)) "jpg"))) ; определяется имя негатива изображения для последующего сохранения в виде "негатив-ХХХХХХХХ.jpg
                                                          (file-jpeg-save RUN-NONINTERACTIVE image layer filename-negative filename-negative 0.9 0 1 1 "" 2 1 0 0) ;сохраняет файл в формате jpeg
                                                          )
                                         )

                                        (if (= format 1) (begin
                                                          (set! filename-negative (string-append inOutFolder DIR-SEPARATOR "негатив-" (morph-filename (car (gimp-image-get-name image)) "tif"))) ; определяется имя негатива изображения для последующего сохранения в виде "негатив-ХХХХХХХХ.tiff              
                                                          (file-tiff-save RUN-NONINTERACTIVE image layer filename-negative filename-negative 0) ; сохраняет негатив фото в формате tiff без компрессии
                                                         )
                                        )
                                     )
                     )
                     
                     ; Приготовление вспомогательного файла imageWhite - копии обрабатываемого для последующего сложения с негативом маски.
            
                     (set! imageWhite (car (gimp-image-duplicate image)))                                   ; создание вспомогательного файла imageWhite - копии обрабатываемого 
                     (set! layerWhite (car (gimp-image-get-active-layer imageWhite)))                       ; задаётся слой созданной копии (layerWhite)
                     (set! drawableWhite (car (gimp-image-get-active-drawable imageWhite)))                 ; задаётся изображение созданной копии (drawableWhite)
                     (set! maskNegativeLayer (car (gimp-layer-new-from-drawable maskNegative imageWhite)))  ; задаётся негативный слой маски maskNegativeLayer  для последующего добавления во вспомогательный файл imageWhite 
                    
                     ; Сложение "сырого" позитива фото (инвертированной копии обрабатываемого файла) с негативом маски
          
                     (gimp-invert drawableWhite)                           ; создание позитива изображения
                     (gimp-layer-set-mode maskNegativeLayer ADDITION-MODE) ; устанавливается режим добавления слоя maskNegativeLayer в imageWhite - "добавление"
                     (gimp-image-add-layer imageWhite maskNegativeLayer -1); слой maskNegativeLayer добавляется в imageWhite
                     (gimp-image-merge-visible-layers imageWhite 1)        ; слои каркаса-изображения imageWhite объединяются в один в соответствии с ранее установленным режимом
                     (set! drawableWhite (car (gimp-image-get-active-drawable imageWhite))) ; определяется изображение drawableWhite из каркаса imageWhite
                     (gimp-invert drawableWhite)                                            ; изображение вспомогательного файла imageWhite инвертируется в негатив для сложения с позитивным изображением обрабатываемого файла image
                    
                     ; Сложение негатива фото с позитивом маски

                     (gimp-layer-set-mode maskPositiveLayer ADDITION-MODE) ;устанавливается режим добавления слоя maskPositiveLayer в image - "добавление"
                     (gimp-image-add-layer image maskPositiveLayer -1)     ;слой maskPositiveLayer добавляется в image
                     (gimp-image-merge-visible-layers image 1)             ;слои каркаса-изображения image объединяются в один в соответствии с ранее установленным режимом
                                
                     ; Сложение позитивного изображения вспомогательного файла drawableWhite с негативным изображением обрабатываемого, инвертирование полученного изображения в позитивное

                     (set! drawableWhite (car (gimp-image-get-active-drawable imageWhite)))
                     (set! layerWhite (car (gimp-layer-new-from-drawable drawableWhite image)));определяется переменная layerWhite изображением drawableWhite в оежиме слоя для последующего добавления в image
                     (gimp-layer-set-mode layerWhite GRAIN-MERGE-MODE)      ;устанавливается режим добавления слоя layerWhite в image - "объединение зерна"
                     (gimp-image-add-layer image layerWhite -1)             ;слой layerWhite добавляется в image
                     (gimp-image-merge-visible-layers image 1)              ;слои каркаса-изображения image объединяются в один в соответствии с ранее установленным режимом
                     (set! drawable (car (gimp-image-get-active-drawable image)))
                     (gimp-invert drawable)                                 ; инвертирование полученного изображения в позитивное
                     
                     ; Дальнейшая обработка полученного изображения

                     (gimp-levels-stretch drawable)                         ; растягивает цвета по динамическому диапазону
                     (if (= bik TRUE) (gimp-brightness-contrast drawable brightness contrast)) ; регулировка яркости и контраста, если выбрано

                     (set! layer (car (gimp-image-get-active-layer image)))
                     
                     
                     ; Сохранение результата обработки

                     (if (= format 0) (begin
                                       (set! filename (string-append inOutFolder DIR-SEPARATOR (morph-filename (car (gimp-image-get-name image)) "jpg"))) ; определяется имя выходного файла и его размещение
                                       (file-jpeg-save RUN-NONINTERACTIVE image layer filename filename 0.9 0 1 1 "" 2 1 0 0) ;сохраняет файл в формате jpeg
                                       )
                     )
                     (if (= format 1) (begin                                       
                                       (set! filename (string-append inOutFolder DIR-SEPARATOR (morph-filename (car (gimp-image-get-name image)) "tif"))) ; определяется имя выходного файла и его размещение
                                       (file-tiff-save RUN-NONINTERACTIVE image layer filename filename 0) ; сохраняет негатив фото в формате tiff без компрессии
                                      )
                     )
                     
                     (gimp-image-delete image)    ; удаление переменной image
                   )
                (set! filelist (cdr filelist))    ; удаление только что обработанного файла из списка filelist путём присвоения списку значения без первого файла
           )
     )
)





(script-fu-register
 "slide-scanner-color-foto-processing"
"<Image>/Script-Fu/Мои скрипты/Обработка цветных негативов"
"Этот скрипт используется при оцифровке негативных цветных изображений фотоплёнки. Он обрабатывает целую папку негативов и сохраняет готовые фотографии в другую папку."
;  В обработку для цветных изображений входит:
;- обрезка ненужных краёв;
;- изменение размера (приведение его к стандартному);
;- переворот изображения (при сканировании со стороны эмульсии);
;- сохранение негатива (по желанию);
;- компенсация оранжевого цвета фотоплёнки методом *Вычитания маски*, а именно, вычитание слоя незафотографированной плёнки - маски из отсканированного негатива для устранения преобладания оранжевого цвета в негативе и синего в позитиве изображения. Делается это в три этапа. К слою изображения негатива добавляется слой изображения позитива маски в режиме добавление, затем к слою изображения позитива добавляется слой изображения негатива маски в режиме добавление и это изображение инвертируется в негатив, и наконец оба полученных изображения суммируются в режиме слияние зерна;
;- инвертирование (при обработке негативов, и, если в фотоаппарате нет функции негатив, сразу инвертирующей отснятое);
;- растяжение динамического диапазона для компенсации затемняющего эффекта фотоплёнки;
;- корректировка яркости и контраста изображения (при необходимости);
;- обесцвечивание (после растяжения динамического диапазона возможно появление какого-либо цвета);
;- сохранение готового изображения в форматах jpeg и tif.
 "Rodionov"
 "Copyleft, use it at your own sweet will"
 "December 11, 2011"
 ""
 SF-DIRNAME "Исходная папка" "/ваша/исходная/папка_по_умолчанию"
 SF-DIRNAME "Папка назначения" "/ваша/назначения/папка_по_умолчанию"
 SF-FILENAME "Плёнка" "/путь/к/маске/maska.JPG"
 SF-ADJUSTMENT "ширина" '(4438 0 6000 1 10 0 SF-SPINNER)
 SF-ADJUSTMENT "высота" '(2959 0 6000 1 10 0 SF-SPINNER)
 SF-ADJUSTMENT "отступ слева" '(70 0 6000 1 10 0 SF-SPINNER)
 SF-ADJUSTMENT "отступ сверху" '(171 0 6000 1 10 0 SF-SPINNER)
 SF-TOGGLE "регулировка яркости и контраста" FALSE                  ; переменная bik
 SF-ADJUSTMENT "яркость" '(0 -100 100 1 10 0 0)
 SF-ADJUSTMENT "контрастность" '(0 -100 100 1 10 0 0)
 SF-TOGGLE "нужны негативы" FALSE                                   ; переменная neg
 SF-TOGGLE "отсканировано со стороны эмульсии" TRUE                 ; переменная flip
 SF-OPTION "сохранить в формате:" '("JPEG" "TIFF")                  ; переменная format
 )


